import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc

try:
    Foundation.NSURLSessionStreamTask

except AttributeError:
    pass

else:

    class TestNSURLSessionStreamTaskHelper(Foundation.NSURLSessionStreamTask):
        def readDataOfMinLength_maxLength_timeout_completionHandler_(
            self, mn, mx, t, c
        ):
            pass

        def writeData_timeout_completionHandler_(self, d, t, c):
            pass


try:
    Foundation.NSURLSessionConfiguration

except NameError:
    pass

else:

    class TestNSURLSessionConfigurationHelper(Foundation.NSURLSessionConfiguration):
        def shouldUseExtendedBackgroundIdleMode(self):
            return 1

        def setShouldUseExtendedBackgroundIdleMode_(self, v):
            pass

        def isDiscretionary(self):
            return 1

        def setDiscretionary_(self, v):
            pass

        def HTTPShouldUsePipelining(self):
            return 1

        def setHTTPShouldUsePipelining_(self, v):
            pass

        def HTTPShouldSetCookies(self):
            return 1

        def setHTTPShouldSetCookies_(self, v):
            pass

        def waitsForConnectivity(self):
            return 1

        def setWaitsForConnectivity_(self, v):
            pass


class TestNSURLSessionHelper(Foundation.NSObject):
    def URLSession_didReceiveChallenge_completionHandler_(self, a, b, c):
        pass

    def URLSession_task_willPerformHTTPRedirection_newRequest_completionHandler_(
        self, a, b, c, d, e
    ):
        pass

    def URLSession_task_didReceiveChallenge_completionHandler_(self, a, b, c, d):
        pass

    def URLSession_task_needNewBodyStream_(self, a, b, c):
        pass

    def URLSession_task_didSendBodyData_totalBytesSent_totalBytesExpectedToSend_(
        self, a, b, c, d, e
    ):
        pass

    def URLSession_dataTask_didReceiveResponse_completionHandler_(self, a, b, c, d):
        pass

    def URLSession_dataTask_willCacheResponse_completionHandler_(self, a, b, c, d):
        pass

    def URLSession_downloadTask_didWriteData_totalBytesWritten_totalBytesExpectedToWrite_(
        self, a, b, c, d, e
    ):
        pass

    def URLSession_downloadTask_didResumeAtOffset_expectedTotalBytes_(self, a, b, c, d):
        pass

    def URLSession_task_willBeginDelayedRequest_completionHandler_(self, a, b, c, d):
        pass


class TestNSURLSessionWebSocketDelegateHelper(Foundation.NSObject):
    def URLSession_webSocketTask_didCloseWithCode_reason_(self, a, b, c, d):
        pass


class TestNSURLSession(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSURLSessionMultipathServiceTypeNone, 0)
        self.assertEqual(Foundation.NSURLSessionMultipathServiceTypeHandover, 1)
        self.assertEqual(Foundation.NSURLSessionMultipathServiceTypeInteractive, 2)
        self.assertEqual(Foundation.NSURLSessionMultipathServiceTypeAggregate, 3)

        self.assertEqual(Foundation.NSURLSessionDelayedRequestContinueLoading, 0)
        self.assertEqual(Foundation.NSURLSessionDelayedRequestUseNewRequest, 1)
        self.assertEqual(Foundation.NSURLSessionDelayedRequestCancel, 2)

        self.assertEqual(Foundation.NSURLSessionWebSocketMessageTypeData, 0)
        self.assertEqual(Foundation.NSURLSessionWebSocketMessageTypeString, 1)

        self.assertEqual(Foundation.NSURLSessionWebSocketCloseCodeInvalid, 0)
        self.assertEqual(Foundation.NSURLSessionWebSocketCloseCodeNormalClosure, 1000)
        self.assertEqual(Foundation.NSURLSessionWebSocketCloseCodeGoingAway, 1001)
        self.assertEqual(Foundation.NSURLSessionWebSocketCloseCodeProtocolError, 1002)
        self.assertEqual(Foundation.NSURLSessionWebSocketCloseCodeUnsupportedData, 1003)
        self.assertEqual(
            Foundation.NSURLSessionWebSocketCloseCodeNoStatusReceived, 1005
        )
        self.assertEqual(Foundation.NSURLSessionWebSocketCloseCodeAbnormalClosure, 1006)
        self.assertEqual(
            Foundation.NSURLSessionWebSocketCloseCodeInvalidFramePayloadData, 1007
        )
        self.assertEqual(Foundation.NSURLSessionWebSocketCloseCodePolicyViolation, 1008)
        self.assertEqual(Foundation.NSURLSessionWebSocketCloseCodeMessageTooBig, 1009)
        self.assertEqual(
            Foundation.NSURLSessionWebSocketCloseCodeMandatoryExtensionMissing, 1010
        )
        self.assertEqual(
            Foundation.NSURLSessionWebSocketCloseCodeInternalServerError, 1011
        )
        self.assertEqual(
            Foundation.NSURLSessionWebSocketCloseCodeTLSHandshakeFailure, 1015
        )

        self.assertEqual(
            Foundation.NSURLSessionTaskMetricsDomainResolutionProtocolUnknown, 0
        )
        self.assertEqual(
            Foundation.NSURLSessionTaskMetricsDomainResolutionProtocolUDP, 1
        )
        self.assertEqual(
            Foundation.NSURLSessionTaskMetricsDomainResolutionProtocolTCP, 2
        )
        self.assertEqual(
            Foundation.NSURLSessionTaskMetricsDomainResolutionProtocolTLS, 3
        )
        self.assertEqual(
            Foundation.NSURLSessionTaskMetricsDomainResolutionProtocolHTTPS, 4
        )

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(Foundation.NSURLSessionTransferSizeUnknown, int)

        self.assertEqual(Foundation.NSURLSessionTaskStateRunning, 0)
        self.assertEqual(Foundation.NSURLSessionTaskStateSuspended, 1)
        self.assertEqual(Foundation.NSURLSessionTaskStateCanceling, 2)
        self.assertEqual(Foundation.NSURLSessionTaskStateCompleted, 3)

        self.assertIsInstance(Foundation.NSURLSessionTaskPriorityDefault, float)
        self.assertIsInstance(Foundation.NSURLSessionTaskPriorityLow, float)
        self.assertIsInstance(Foundation.NSURLSessionTaskPriorityHigh, float)

        self.assertEqual(Foundation.NSURLSessionAuthChallengeUseCredential, 0)
        self.assertEqual(Foundation.NSURLSessionAuthChallengePerformDefaultHandling, 1)
        self.assertEqual(
            Foundation.NSURLSessionAuthChallengeCancelAuthenticationChallenge, 2
        )
        self.assertEqual(Foundation.NSURLSessionAuthChallengeRejectProtectionSpace, 3)

        self.assertEqual(Foundation.NSURLSessionResponseCancel, 0)
        self.assertEqual(Foundation.NSURLSessionResponseAllow, 1)
        self.assertEqual(Foundation.NSURLSessionResponseBecomeDownload, 2)
        self.assertEqual(Foundation.NSURLSessionResponseBecomeStream, 3)

        self.assertIsInstance(Foundation.NSURLSessionDownloadTaskResumeData, str)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertEqual(Foundation.NSURLSessionTaskMetricsResourceFetchTypeUnknown, 0)
        self.assertEqual(
            Foundation.NSURLSessionTaskMetricsResourceFetchTypeNetworkLoad, 1
        )
        self.assertEqual(
            Foundation.NSURLSessionTaskMetricsResourceFetchTypeServerPush, 2
        )
        self.assertEqual(
            Foundation.NSURLSessionTaskMetricsResourceFetchTypeLocalCache, 3
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsBlock(
            Foundation.NSURLSession.resetWithCompletionHandler_, 0, b"v"
        )
        self.assertArgIsBlock(
            Foundation.NSURLSession.flushWithCompletionHandler_, 0, b"v"
        )
        self.assertArgIsBlock(
            Foundation.NSURLSession.getTasksWithCompletionHandler_, 0, b"v@@@"
        )

        self.assertArgIsBlock(
            Foundation.NSURLSession.dataTaskWithRequest_completionHandler_, 1, b"v@@@"
        )
        self.assertArgIsBlock(
            Foundation.NSURLSession.dataTaskWithURL_completionHandler_, 1, b"v@@@"
        )
        self.assertArgIsBlock(
            Foundation.NSURLSession.uploadTaskWithRequest_fromFile_completionHandler_,
            2,
            b"v@@@",
        )
        self.assertArgIsBlock(
            Foundation.NSURLSession.uploadTaskWithRequest_fromData_completionHandler_,
            2,
            b"v@@@",
        )
        self.assertArgIsBlock(
            Foundation.NSURLSession.downloadTaskWithURL_completionHandler_, 1, b"v@@@"
        )
        self.assertArgIsBlock(
            Foundation.NSURLSession.downloadTaskWithResumeData_completionHandler_,
            1,
            b"v@@@",
        )

        self.assertArgIsBlock(
            Foundation.NSURLSessionDownloadTask.cancelByProducingResumeData_, 0, b"v@"
        )

        self.assertResultIsBOOL(
            TestNSURLSessionConfigurationHelper.waitsForConnectivity
        )
        self.assertArgIsBOOL(
            TestNSURLSessionConfigurationHelper.setWaitsForConnectivity_, 0
        )

        self.assertResultIsBOOL(TestNSURLSessionConfigurationHelper.isDiscretionary)
        self.assertArgIsBOOL(TestNSURLSessionConfigurationHelper.setDiscretionary_, 0)

        self.assertResultIsBOOL(
            TestNSURLSessionConfigurationHelper.HTTPShouldUsePipelining
        )
        self.assertArgIsBOOL(
            TestNSURLSessionConfigurationHelper.setHTTPShouldUsePipelining_, 0
        )

        self.assertResultIsBOOL(
            TestNSURLSessionConfigurationHelper.HTTPShouldSetCookies
        )
        self.assertArgIsBOOL(
            TestNSURLSessionConfigurationHelper.setHTTPShouldSetCookies_, 0
        )

        self.assertArgIsBlock(
            TestNSURLSessionHelper.URLSession_didReceiveChallenge_completionHandler_,
            2,
            b"v" + objc._C_NSUInteger + b"@",
        )
        self.assertArgIsBlock(
            TestNSURLSessionHelper.URLSession_task_willPerformHTTPRedirection_newRequest_completionHandler_,  # noqa: B950
            4,
            b"v@",
        )
        self.assertArgIsBlock(
            TestNSURLSessionHelper.URLSession_task_didReceiveChallenge_completionHandler_,
            3,
            b"v" + objc._C_NSUInteger + b"@",
        )
        self.assertArgIsBlock(
            TestNSURLSessionHelper.URLSession_task_needNewBodyStream_, 2, b"v@"
        )

        self.assertArgHasType(
            TestNSURLSessionHelper.URLSession_task_didSendBodyData_totalBytesSent_totalBytesExpectedToSend_,  # noqa: B950
            2,
            objc._C_LNG_LNG,
        )
        self.assertArgHasType(
            TestNSURLSessionHelper.URLSession_task_didSendBodyData_totalBytesSent_totalBytesExpectedToSend_,  # noqa: B950
            3,
            objc._C_LNG_LNG,
        )
        self.assertArgHasType(
            TestNSURLSessionHelper.URLSession_task_didSendBodyData_totalBytesSent_totalBytesExpectedToSend_,  # noqa: B950
            4,
            objc._C_LNG_LNG,
        )

        self.assertArgIsBlock(
            TestNSURLSessionHelper.URLSession_dataTask_didReceiveResponse_completionHandler_,
            3,
            b"v" + objc._C_NSUInteger,
        )
        self.assertArgIsBlock(
            TestNSURLSessionHelper.URLSession_dataTask_willCacheResponse_completionHandler_,
            3,
            b"v@",
        )
        self.assertArgIsBlock(
            TestNSURLSessionHelper.URLSession_dataTask_willCacheResponse_completionHandler_,
            3,
            b"v@",
        )

        self.assertArgHasType(
            TestNSURLSessionHelper.URLSession_downloadTask_didWriteData_totalBytesWritten_totalBytesExpectedToWrite_,  # noqa: B950
            2,
            objc._C_LNG_LNG,
        )
        self.assertArgHasType(
            TestNSURLSessionHelper.URLSession_downloadTask_didWriteData_totalBytesWritten_totalBytesExpectedToWrite_,  # noqa: B950
            3,
            objc._C_LNG_LNG,
        )
        self.assertArgHasType(
            TestNSURLSessionHelper.URLSession_downloadTask_didWriteData_totalBytesWritten_totalBytesExpectedToWrite_,  # noqa: B950
            4,
            objc._C_LNG_LNG,
        )

        self.assertArgHasType(
            TestNSURLSessionHelper.URLSession_downloadTask_didResumeAtOffset_expectedTotalBytes_,  # noqa: B950
            2,
            objc._C_LNG_LNG,
        )
        self.assertArgHasType(
            TestNSURLSessionHelper.URLSession_downloadTask_didResumeAtOffset_expectedTotalBytes_,  # noqa: B950
            3,
            objc._C_LNG_LNG,
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertArgIsBlock(
            Foundation.NSURLSession.getAllTasksWithCompletionHandler_, 0, b"v@"
        )
        self.assertArgIsBlock(
            TestNSURLSessionStreamTaskHelper.readDataOfMinLength_maxLength_timeout_completionHandler_,  # noqa: B950
            3,
            b"v@Z@",
        )
        self.assertArgIsBlock(
            TestNSURLSessionStreamTaskHelper.writeData_timeout_completionHandler_,
            2,
            b"v@",
        )

        self.assertResultIsBOOL(
            TestNSURLSessionConfigurationHelper.shouldUseExtendedBackgroundIdleMode
        )
        self.assertArgIsBOOL(
            TestNSURLSessionConfigurationHelper.setShouldUseExtendedBackgroundIdleMode_,
            0,
        )

    @min_os_level("10.15")
    def testMethods10_12(self):
        self.assertResultIsBOOL(
            Foundation.NSURLSessionTaskTransactionMetrics.isProxyConnection
        )
        self.assertResultIsBOOL(
            Foundation.NSURLSessionTaskTransactionMetrics.isReusedConnection
        )

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBlock(
            TestNSURLSessionHelper.URLSession_task_willBeginDelayedRequest_completionHandler_,
            3,
            b"v" + objc._C_NSInteger + b"@",
        )

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertArgIsBlock(
            Foundation.NSURLSessionWebSocketTask.sendMessage_completionHandler_,
            1,
            b"v@",
        )
        self.assertArgIsBlock(
            Foundation.NSURLSessionWebSocketTask.receiveMessageWithCompletionHandler_,
            0,
            b"v@@",
        )
        self.assertArgIsBlock(
            Foundation.NSURLSessionWebSocketTask.sendPingWithPongReceiveHandler_,
            0,
            b"v@",
        )

        self.assertResultIsBOOL(
            Foundation.NSURLSessionTaskTransactionMetrics.isCellular
        )
        self.assertResultIsBOOL(
            Foundation.NSURLSessionTaskTransactionMetrics.isExpensive
        )
        self.assertResultIsBOOL(
            Foundation.NSURLSessionTaskTransactionMetrics.isConstrained
        )
        self.assertResultIsBOOL(
            Foundation.NSURLSessionTaskTransactionMetrics.isMultipath
        )

    @min_os_level("11.0")
    def testMethods11_0(self):
        self.assertResultIsBOOL(
            Foundation.NSURLSessionConfiguration.sessionSendsLaunchEvents
        )
        self.assertArgIsBOOL(
            Foundation.NSURLSessionConfiguration.setSessionSendsLaunchEvents_, 0
        )

    @min_sdk_level("10.12")
    def testProtocols10_12(self):
        objc.protocolNamed("NSURLSessionTaskDelegate")
        objc.protocolNamed("NSURLSessionDataDelegate")
        objc.protocolNamed("NSURLSessionDownloadDelegate")

    @min_sdk_level("10.11")
    def testProtocols10_11(self):
        objc.protocolNamed("NSURLSessionStreamDelegate")

    def test_protocol_methods10_15(self):
        self.assertArgHasType(
            TestNSURLSessionWebSocketDelegateHelper.URLSession_webSocketTask_didCloseWithCode_reason_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )



import AppKit
import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSBundle(TestCase):
    def testMethods(self):
        b = AppKit.NSBundle.mainBundle()
        # Test on an instance because AppKit.NSBundle has class methods
        # that interfere with this test
        self.assertResultIsBOOL(b.load)
        self.assertResultIsBOOL(b.isLoaded)
        self.assertResultIsBOOL(b.unload)

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(AppKit.NSBundle.loadNibNamed_owner_topLevelObjects_)
        self.assertArgIsOut(AppKit.NSBundle.loadNibNamed_owner_topLevelObjects_, 2)

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(AppKit.NSBundle.preflightAndReturnError_)
        self.assertArgIsOut(AppKit.NSBundle.preflightAndReturnError_, 0)
        self.assertResultIsBOOL(AppKit.NSBundle.loadAndReturnError_)
        self.assertArgIsOut(AppKit.NSBundle.loadAndReturnError_, 0)

    def testConstants(self):
        self.assertEqual(AppKit.NSBundleExecutableArchitectureI386, 0x00000007)
        self.assertEqual(AppKit.NSBundleExecutableArchitecturePPC, 0x00000012)
        self.assertEqual(AppKit.NSBundleExecutableArchitectureX86_64, 0x01000007)
        self.assertEqual(AppKit.NSBundleExecutableArchitecturePPC64, 0x01000012)
        self.assertEqual(AppKit.NSBundleExecutableArchitectureARM64, 0x0100000C)

        self.assertIsInstance(AppKit.NSBundleDidLoadNotification, str)
        self.assertIsInstance(AppKit.NSLoadedClasses, str)

    def testDefines(self):
        self.assertHasAttr(Foundation, "NSLocalizedString")
        self.assertHasAttr(Foundation, "NSLocalizedStringFromTable")
        self.assertHasAttr(Foundation, "NSLocalizedStringFromTableInBundle")
        self.assertHasAttr(Foundation, "NSLocalizedStringWithDefaultValue")

    def testDefines12_0(self):
        # XXX: these should only be available on macOS 12
        self.assertHasAttr(Foundation, "NSLocalizedAttributedString")
        self.assertHasAttr(Foundation, "NSLocalizedAttributedStringFromTable")
        self.assertHasAttr(Foundation, "NSLocalizedAttributedStringFromTableInBundle")
        self.assertHasAttr(Foundation, "NSLocalizedAttributedStringWithDefaultValue")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSWindowTabGroup(TestCase):
    @min_os_level("10.13")
    def testMethods(self):
        # Use subclass for testing because these methods aren't present
        # on the main class.

        class TestNSWindowTabGroupHelper(AppKit.NSWindowTabGroup):
            def isOverviewVisible(self):
                return 1

            def setOverviewVisible_(self, v):
                pass

            def isTabBarVisible(self):
                return 1

            def setTabBarVisible_(self, v):
                pass

        self.assertResultIsBOOL(TestNSWindowTabGroupHelper.isOverviewVisible)
        self.assertArgIsBOOL(TestNSWindowTabGroupHelper.setOverviewVisible_, 0)

        self.assertResultIsBOOL(TestNSWindowTabGroupHelper.isTabBarVisible)
        self.assertArgIsBOOL(TestNSWindowTabGroupHelper.setTabBarVisible_, 0)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSOrderedCollectionChange(TestCase):
    def test_constants(self):
        self.assertEqual(Foundation.NSCollectionChangeInsert, 0)
        self.assertEqual(Foundation.NSCollectionChangeRemove, 1)



import os
import objc

import Foundation
from PyObjCTools.TestSupport import TestCase, max_os_level


class TestNSJavaSetup(TestCase):
    @max_os_level("10.5")
    def testConstants(self):
        self.assertIsInstance(Foundation.NSJavaClasses, str)
        self.assertIsInstance(Foundation.NSJavaRoot, str)
        self.assertIsInstance(Foundation.NSJavaPath, str)
        self.assertIsInstance(Foundation.NSJavaUserPath, str)
        self.assertIsInstance(Foundation.NSJavaLibraryPath, str)
        self.assertIsInstance(Foundation.NSJavaOwnVirtualMachine, str)
        self.assertIsInstance(Foundation.NSJavaPathSeparator, str)
        self.assertIsInstance(Foundation.NSJavaWillSetupVirtualMachineNotification, str)
        self.assertIsInstance(Foundation.NSJavaDidSetupVirtualMachineNotification, str)
        self.assertIsInstance(
            Foundation.NSJavaWillCreateVirtualMachineNotification, str
        )
        self.assertIsInstance(Foundation.NSJavaDidCreateVirtualMachineNotification, str)

    @max_os_level("10.5")
    def testFunctions(self):
        v = Foundation.NSJavaNeedsVirtualMachine({})
        self.assertIs(v, False)
        v = Foundation.NSJavaProvidesClasses({})
        self.assertIs(v, False)
        v = Foundation.NSJavaNeedsToLoadClasses({})
        self.assertIs(v, False)
        vm = Foundation.NSJavaSetup({})
        self.assertIsInstance(vm, objc.objc_object)
        v = Foundation.NSJavaSetupVirtualMachine()
        self.assertIsInstance(v, objc.objc_object)
        v = Foundation.NSJavaObjectNamedInPath("java.lang.Object", None)
        self.assertIsInstance(v, objc.objc_object)
        v, vm = Foundation.NSJavaClassesFromPath(None, ["java.lang.Object"], True, None)
        self.assertIsInstance(v, Foundation.NSArray)
        self.assertEqual(len(v), 1)
        self.assertIsInstance(vm, objc.objc_object)
        v, vm = Foundation.NSJavaClassesForBundle(
            Foundation.NSBundle.mainBundle(), True, None
        )
        self.assertIsInstance(v, Foundation.NSArray)
        self.assertEqual(len(v), 0)
        self.assertIsInstance(vm, objc.objc_object)
        vm = Foundation.NSJavaBundleSetup(Foundation.NSBundle.mainBundle(), {})
        self.assertIsInstance(vm, objc.objc_object)
        # FIXME: Foundation.NSJavaBundleCleanup gives an exception
        # This seems to be related to the way we call these APIs and I don't
        # plan to fix is (there is no problem with PyObjC or the Foundation
        # wrappers)
        fd = os.dup(2)
        x = os.open("/dev/null", os.O_WRONLY)
        os.dup2(x, 2)
        os.close(x)
        try:
            try:
                Foundation.NSJavaBundleCleanup(Foundation.NSBundle.mainBundle(), {})
            except ValueError:
                pass
        finally:
            os.dup2(fd, 2)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSNetServicesHelper(Foundation.NSObject):
    def netServiceBrowser_didFindDomain_moreComing_(self, a, b, c):
        pass

    def netServiceBrowser_didFindService_moreComing_(self, a, b, c):
        pass

    def netServiceBrowser_didRemoveDomain_moreComing_(self, a, b, c):
        pass

    def netServiceBrowser_didRemoveService_moreComing_(self, a, b, c):
        pass


class TestNSNetservices(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSNetServicesErrorCode, str)
        self.assertIsInstance(Foundation.NSNetServicesErrorDomain, str)
        self.assertEqual(Foundation.NSNetServicesUnknownError, -72000)
        self.assertEqual(Foundation.NSNetServicesCollisionError, -72001)
        self.assertEqual(Foundation.NSNetServicesNotFoundError, -72002)
        self.assertEqual(Foundation.NSNetServicesActivityInProgress, -72003)
        self.assertEqual(Foundation.NSNetServicesBadArgumentError, -72004)
        self.assertEqual(Foundation.NSNetServicesCancelledError, -72005)
        self.assertEqual(Foundation.NSNetServicesInvalidError, -72006)
        self.assertEqual(Foundation.NSNetServicesTimeoutError, -72007)
        self.assertEqual(
            Foundation.NSNetServicesMissingRequiredConfigurationError, -72008
        )
        self.assertEqual(Foundation.NSNetServiceNoAutoRename, 1)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertEqual(Foundation.NSNetServiceListenForConnections, 1 << 1)

    def testOutput(self):
        o = Foundation.NSNetService.alloc().initWithDomain_type_name_port_(
            "", "_http._tcp.", "", 0
        )

        m = o.getInputStream_outputStream_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"Z")
        self.assertEqual(m["arguments"][2]["type"], b"o^@")
        self.assertEqual(m["arguments"][3]["type"], b"o^@")

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSNetService.getInputStream_outputStream_)
        self.assertArgIsOut(Foundation.NSNetService.getInputStream_outputStream_, 0)
        self.assertArgIsOut(Foundation.NSNetService.getInputStream_outputStream_, 1)
        self.assertResultIsBOOL(Foundation.NSNetService.setTXTRecordData_)

        self.assertArgIsBOOL(
            TestNSNetServicesHelper.netServiceBrowser_didFindDomain_moreComing_, 2
        )
        self.assertArgIsBOOL(
            TestNSNetServicesHelper.netServiceBrowser_didFindService_moreComing_, 2
        )
        self.assertArgIsBOOL(
            TestNSNetServicesHelper.netServiceBrowser_didRemoveDomain_moreComing_, 2
        )
        self.assertArgIsBOOL(
            TestNSNetServicesHelper.netServiceBrowser_didRemoveService_moreComing_, 2
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(Foundation.NSNetService.includesPeerToPeer)
        self.assertArgIsBOOL(Foundation.NSNetService.setIncludesPeerToPeer_, 0)

        self.assertResultIsBOOL(Foundation.NSNetServiceBrowser.includesPeerToPeer)
        self.assertArgIsBOOL(Foundation.NSNetServiceBrowser.setIncludesPeerToPeer_, 0)

    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSNetServiceDelegate")
        objc.protocolNamed("NSNetServiceBrowserDelegate")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level


class TestNSTouch(TestCase):
    @min_os_level("10.6")
    def testConstants(self):
        self.assertEqual(AppKit.NSTouchPhaseBegan, 1 << 0)
        self.assertEqual(AppKit.NSTouchPhaseMoved, 1 << 1)
        self.assertEqual(AppKit.NSTouchPhaseStationary, 1 << 2)
        self.assertEqual(AppKit.NSTouchPhaseEnded, 1 << 3)
        self.assertEqual(AppKit.NSTouchPhaseCancelled, 1 << 4)
        self.assertEqual(
            AppKit.NSTouchPhaseTouching,
            AppKit.NSTouchPhaseBegan
            | AppKit.NSTouchPhaseMoved
            | AppKit.NSTouchPhaseStationary,
        )
        self.assertEqual(AppKit.NSTouchPhaseAny, 0xFFFFFFFFFFFFFFFF)

        # 10.12
        self.assertEqual(AppKit.NSTouchTypeDirect, 0)
        self.assertEqual(AppKit.NSTouchTypeIndirect, 1)
        self.assertEqual(AppKit.NSTouchTypeMaskDirect, 1 << 0)
        self.assertEqual(AppKit.NSTouchTypeMaskIndirect, 1 << 1)

    @min_os_level("10.6")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTouch.isResting)
        self.assertResultHasType(AppKit.NSTouch.deviceSize, AppKit.NSSize.__typestr__)

    @min_sdk_level("10.12")
    def testFunctions10_12(self):
        self.assertEqual(
            AppKit.NSTouchTypeMaskFromType(AppKit.NSTouchTypeDirect),
            AppKit.NSTouchTypeMaskDirect,
        )
        self.assertEqual(
            AppKit.NSTouchTypeMaskFromType(AppKit.NSTouchTypeIndirect),
            AppKit.NSTouchTypeMaskIndirect,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSPrinter(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSPrinterTableOK, 0)
        self.assertEqual(AppKit.NSPrinterTableNotFound, 1)
        self.assertEqual(AppKit.NSPrinterTableError, 2)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSPrinter.isKey_inTable_)
        self.assertResultIsBOOL(AppKit.NSPrinter.booleanForKey_inTable_)
        self.assertResultIsBOOL(AppKit.NSPrinter.acceptsBinary)
        self.assertResultIsBOOL(AppKit.NSPrinter.isColor)
        self.assertResultIsBOOL(AppKit.NSPrinter.isFontAvailable_)
        self.assertResultIsBOOL(AppKit.NSPrinter.isOutputStackInReverseOrder)
        self.assertArgIsBOOL(
            AppKit.NSPrinter.printerWithName_domain_includeUnavailable_, 2
        )



import Cocoa
from objc import super

RowSelectedNotification = "RowSelectedNotification"


class SelectionNotifyMatrix(Cocoa.NSMatrix):
    def mouseDown_(self, theEvent):
        super().mouseDown_(theEvent)

        row = self.selectedRow()
        if row != -1:
            Cocoa.NSNotificationCenter.defaultCenter().postNotificationName_object_userInfo_(
                RowSelectedNotification, self, None
            )

    def selectCellAtRow_column_(self, row, col):
        super().selectCellAtRow_column_(row, col)

        Cocoa.NSNotificationCenter.defaultCenter().postNotificationName_object_userInfo_(
            RowSelectedNotification, self, None
        )



#
#  FontNameToDisplayNameTransformer.py
#  ControlledPreferences
#
#  Converted by u.fiedler on 04.02.05.
#  with great help from Bob Ippolito - Thank you Bob!
#
#  The original version was written in Objective-C by Malcolm Crawford
#  at http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

from Cocoa import NSFont, NSString, NSValueTransformer


class FontNameToDisplayNameTransformer(NSValueTransformer):
    """
    Takes as input the fontName of a font as stored in user defaults,
    returns the displayed font name of the font to show to the user.
    """

    @classmethod
    def transformedValueClass(cls):
        return NSString

    @classmethod
    def allowsReverseTransformation(cls):
        return False

    def transformedValue_(self, aValue):
        font = NSFont.fontWithName_size_(aValue, 12)
        return font.displayName()



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTask(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSTask.suspend)
        self.assertResultIsBOOL(Foundation.NSTask.resume)
        self.assertResultIsBOOL(Foundation.NSTask.isRunning)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsBlock(Foundation.NSTask.setTerminationHandler_, 0, b"v@")
        self.assertResultIsBlock(Foundation.NSTask.terminationHandler, b"v@")

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsOut(
            Foundation.NSTask.launchedTaskWithExecutableURL_arguments_error_terminationHandler_,
            2,
        )
        self.assertArgIsBlock(
            Foundation.NSTask.launchedTaskWithExecutableURL_arguments_error_terminationHandler_,
            3,
            b"v@",
        )

    def testConstants(self):
        self.assertIsInstance(Foundation.NSTaskDidTerminateNotification, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(Foundation.NSTaskTerminationReasonExit, 1)
        self.assertEqual(Foundation.NSTaskTerminationReasonUncaughtSignal, 2)



"""
Script for building the example.

Usage:
    python2 setup.py py2app
"""
from setuptools import setup

plist = {"NSPrincipalClass": "HotKeyApp"}


setup(
    app=["HotKey.py"],
    data_files=["English.lproj"],
    options={"py2app": {"plist": plist}},
    setup_requires=["py2app", "pyobjc-framework-Cocoa"],
)



"""
Wrappers for the core Cocoa frameworks: CoreFoundation, Foundation and
AppKit.

These wrappers don't include documentation, please check Apple's documentation
for information on how to use these frameworks and PyObjC's documentation
for general tips and tricks regarding the translation between Python
and (Objective-)C frameworks
"""
import os

from pyobjc_setup import Extension, setup

VERSION = "8.0"

setup(
    name="pyobjc-framework-Cocoa",
    description="Wrappers for the Cocoa frameworks on macOS",
    packages=["Cocoa", "CoreFoundation", "Foundation", "AppKit", "PyObjCTools"],
    namespace_packages=["PyObjCTools"],
    ext_modules=[
        # CoreFoundation
        Extension(
            "CoreFoundation._inlines",
            ["Modules/_CoreFoundation_inlines.m"],
            extra_link_args=["-framework", "CoreFoundation"],
        ),
        Extension(
            "CoreFoundation._CoreFoundation",
            ["Modules/_CoreFoundation.m"],
            extra_link_args=["-framework", "CoreFoundation"],
            depends=[
                os.path.join("Modules", fn)
                for fn in os.listdir("Modules")
                if fn.startswith("_CoreFoundation")
            ],
        ),
        # Foundation
        Extension(
            "Foundation._inlines",
            ["Modules/_Foundation_inlines.m"],
            extra_link_args=["-framework", "Foundation"],
        ),
        Extension(
            "Foundation._Foundation",
            ["Modules/_Foundation.m"],
            extra_link_args=["-framework", "Foundation"],
            depends=[
                os.path.join("Modules", fn)
                for fn in os.listdir("Modules")
                if fn.startswith("_Foundation")
            ],
        ),
        # AppKit
        Extension(
            "AppKit._inlines",
            ["Modules/_AppKit_inlines.m"],
            extra_link_args=["-framework", "AppKit"],
        ),
        Extension(
            "AppKit._AppKit",
            ["Modules/_AppKit.m"],
            extra_link_args=["-framework", "AppKit"],
            depends=[
                os.path.join("Modules", fn)
                for fn in os.listdir("Modules")
                if fn.startswith("_AppKit")
            ],
        ),
        #
        # Test support
        #
        Extension(
            "PyObjCTest.testhelper",
            ["Modules/testhelper.m"],
            extra_link_args=["-framework", "Foundation"],
        ),
    ],
    version=VERSION,
    install_requires=["pyobjc-core>=" + VERSION],
    long_description=__doc__,
)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSCharacterSet(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSOpenStepUnicodeReservedBase, 0xF400)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSCharacterSet.characterIsMember_)
        self.assertResultIsBOOL(Foundation.NSCharacterSet.longCharacterIsMember_)
        self.assertResultIsBOOL(Foundation.NSCharacterSet.isSupersetOfSet_)
        self.assertResultIsBOOL(Foundation.NSCharacterSet.hasMemberInPlane_)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSAlertHelper(AppKit.NSObject):
    def alertShowHelp_(self, alert):
        return 1


class TestNSAlert(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSWarningAlertStyle, 0)
        self.assertEqual(AppKit.NSInformationalAlertStyle, 1)
        self.assertEqual(AppKit.NSCriticalAlertStyle, 2)

        self.assertEqual(AppKit.NSAlertStyleWarning, 0)
        self.assertEqual(AppKit.NSAlertStyleInformational, 1)
        self.assertEqual(AppKit.NSAlertStyleCritical, 2)

        self.assertEqual(AppKit.NSAlertFirstButtonReturn, 1000)
        self.assertEqual(AppKit.NSAlertSecondButtonReturn, 1001)
        self.assertEqual(AppKit.NSAlertThirdButtonReturn, 1002)

    def testMethods(self):
        self.assertArgIsPrintf(
            AppKit.NSAlert.alertWithMessageText_defaultButton_alternateButton_otherButton_informativeTextWithFormat_,  # noqa: B950
            4,
        )
        self.assertArgIsSEL(
            AppKit.NSAlert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo_,
            2,
            b"v@:@" + objc._C_NSInteger + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSAlert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo_,
            3,
            b"^v",
        )

        self.assertResultIsBOOL(AppKit.NSAlert.showsHelp)
        self.assertArgIsBOOL(AppKit.NSAlert.setShowsHelp_, 0)

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(AppKit.NSAlert.showsSuppressionButton)
        self.assertArgIsBOOL(AppKit.NSAlert.setShowsSuppressionButton_, 0)

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertArgIsBlock(
            AppKit.NSAlert.beginSheetModalForWindow_completionHandler_,
            1,
            b"v" + objc._C_NSInteger,
        )

    def testProtocols(self):
        self.assertResultIsBOOL(TestNSAlertHelper.alertShowHelp_)

    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSAlertDelegate")



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSPortNameServer(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSPortNameServer.registerPort_name_)
        self.assertResultIsBOOL(Foundation.NSPortNameServer.removePortForName_)

        self.assertResultIsBOOL(Foundation.NSMachBootstrapServer.registerPort_name_)

        self.assertResultIsBOOL(Foundation.NSSocketPortNameServer.registerPort_name_)
        self.assertResultIsBOOL(Foundation.NSSocketPortNameServer.removePortForName_)
        self.assertResultIsBOOL(
            Foundation.NSSocketPortNameServer.registerPort_name_nameServerPortNumber_
        )



import objc
import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSWorkspace(TestCase):
    def testInfoForFile(self):
        ws = AppKit.NSWorkspace.sharedWorkspace()

        # A method with 2 output parameters, this means the result
        # is a tuple with 3 elements (return value, param1, param2)
        res = ws.getInfoForFile_application_type_("/", None, None)
        self.assertIsInstance(res, tuple)
        self.assertEqual(len(res), 3)
        self.assertEqual(res[0], 1)
        self.assertEqual(res[1], "/System/Library/CoreServices/Finder.app")
        self.assertEqual(res[2], "")

    def testConstants(self):
        self.assertEqual(AppKit.NSWorkspaceLaunchAndPrint, 0x00000002)
        self.assertEqual(AppKit.NSWorkspaceLaunchWithErrorPresentation, 0x00000040)
        self.assertEqual(AppKit.NSWorkspaceLaunchInhibitingBackgroundOnly, 0x00000080)
        self.assertEqual(AppKit.NSWorkspaceLaunchWithoutAddingToRecents, 0x00000100)
        self.assertEqual(AppKit.NSWorkspaceLaunchWithoutActivation, 0x00000200)
        self.assertEqual(AppKit.NSWorkspaceLaunchAsync, 0x00010000)
        self.assertEqual(AppKit.NSWorkspaceLaunchAllowingClassicStartup, 0x00020000)
        self.assertEqual(AppKit.NSWorkspaceLaunchPreferringClassic, 0x00040000)
        self.assertEqual(AppKit.NSWorkspaceLaunchNewInstance, 0x00080000)
        self.assertEqual(AppKit.NSWorkspaceLaunchAndHide, 0x00100000)
        self.assertEqual(AppKit.NSWorkspaceLaunchAndHideOthers, 0x00200000)
        self.assertEqual(AppKit.NSWorkspaceLaunchDefault, AppKit.NSWorkspaceLaunchAsync)

        self.assertEqual(AppKit.NSExcludeQuickDrawElementsIconCreationOption, 1 << 1)
        self.assertEqual(AppKit.NSExclude10_4ElementsIconCreationOption, 1 << 2)

        self.assertIsInstance(AppKit.NSWorkspaceDidLaunchApplicationNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceDidMountNotification, str)
        self.assertIsInstance(
            AppKit.NSWorkspaceDidPerformFileOperationNotification, str
        )
        self.assertIsInstance(
            AppKit.NSWorkspaceDidTerminateApplicationNotification, str
        )
        self.assertIsInstance(AppKit.NSWorkspaceDidUnmountNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceWillLaunchApplicationNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceWillPowerOffNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceWillUnmountNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceWillSleepNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceDidWakeNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceSessionDidBecomeActiveNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceSessionDidResignActiveNotification, str)
        self.assertIsInstance(AppKit.NSPlainFileType, str)
        self.assertIsInstance(AppKit.NSDirectoryFileType, str)
        self.assertIsInstance(AppKit.NSApplicationFileType, str)
        self.assertIsInstance(AppKit.NSFilesystemFileType, str)
        self.assertIsInstance(AppKit.NSShellCommandFileType, str)
        self.assertIsInstance(AppKit.NSWorkspaceMoveOperation, str)
        self.assertIsInstance(AppKit.NSWorkspaceCopyOperation, str)
        self.assertIsInstance(AppKit.NSWorkspaceLinkOperation, str)
        self.assertIsInstance(AppKit.NSWorkspaceCompressOperation, str)
        self.assertIsInstance(AppKit.NSWorkspaceDecompressOperation, str)
        self.assertIsInstance(AppKit.NSWorkspaceEncryptOperation, str)
        self.assertIsInstance(AppKit.NSWorkspaceDecryptOperation, str)
        self.assertIsInstance(AppKit.NSWorkspaceDestroyOperation, str)
        self.assertIsInstance(AppKit.NSWorkspaceRecycleOperation, str)
        self.assertIsInstance(AppKit.NSWorkspaceDuplicateOperation, str)

        self.assertEqual(AppKit.NSWorkspaceAuthorizationTypeCreateSymbolicLink, 0)
        self.assertEqual(AppKit.NSWorkspaceAuthorizationTypeSetAttributes, 1)
        self.assertEqual(AppKit.NSWorkspaceAuthorizationTypeReplaceFile, 2)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(AppKit.NSWorkspaceDesktopImageScalingKey, str)
        self.assertIsInstance(AppKit.NSWorkspaceDesktopImageAllowClippingKey, str)
        self.assertIsInstance(AppKit.NSWorkspaceDesktopImageFillColorKey, str)
        self.assertIsInstance(AppKit.NSWorkspaceApplicationKey, str)
        self.assertIsInstance(AppKit.NSWorkspaceDidHideApplicationNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceDidUnhideApplicationNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceDidActivateApplicationNotification, str)
        self.assertIsInstance(
            AppKit.NSWorkspaceDidDeactivateApplicationNotification, str
        )
        self.assertIsInstance(AppKit.NSWorkspaceVolumeLocalizedNameKey, str)
        self.assertIsInstance(AppKit.NSWorkspaceVolumeURLKey, str)
        self.assertIsInstance(AppKit.NSWorkspaceVolumeOldLocalizedNameKey, str)
        self.assertIsInstance(AppKit.NSWorkspaceVolumeOldURLKey, str)
        self.assertIsInstance(AppKit.NSWorkspaceDidRenameVolumeNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceScreensDidSleepNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceScreensDidWakeNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceDidChangeFileLabelsNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceActiveSpaceDidChangeNotification, str)
        self.assertIsInstance(AppKit.NSWorkspaceLaunchConfigurationAppleEvent, str)
        self.assertIsInstance(AppKit.NSWorkspaceLaunchConfigurationArguments, str)
        self.assertIsInstance(AppKit.NSWorkspaceLaunchConfigurationEnvironment, str)
        self.assertIsInstance(AppKit.NSWorkspaceLaunchConfigurationArchitecture, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSWorkspace.openFile_)
        self.assertResultIsBOOL(AppKit.NSWorkspace.openFile_withApplication_)
        self.assertResultIsBOOL(
            AppKit.NSWorkspace.openFile_withApplication_andDeactivate_
        )
        self.assertArgIsBOOL(
            AppKit.NSWorkspace.openFile_withApplication_andDeactivate_, 2
        )
        self.assertResultIsBOOL(AppKit.NSWorkspace.openTempFile_)
        self.assertResultIsBOOL(AppKit.NSWorkspace.openFile_fromImage_at_inView_)
        self.assertResultIsBOOL(AppKit.NSWorkspace.openURL_)
        self.assertResultIsBOOL(AppKit.NSWorkspace.launchApplication_)
        self.assertResultIsBOOL(
            AppKit.NSWorkspace.launchApplication_showIcon_autolaunch_
        )
        self.assertArgIsBOOL(
            AppKit.NSWorkspace.launchApplication_showIcon_autolaunch_, 1
        )
        self.assertArgIsBOOL(
            AppKit.NSWorkspace.launchApplication_showIcon_autolaunch_, 2
        )
        self.assertResultIsBOOL(AppKit.NSWorkspace.selectFile_inFileViewerRootedAtPath_)
        self.assertResultIsBOOL(AppKit.NSWorkspace.fileSystemChanged)
        self.assertResultIsBOOL(AppKit.NSWorkspace.userDefaultsChanged)
        self.assertResultIsBOOL(AppKit.NSWorkspace.getInfoForFile_application_type_)
        self.assertArgIsOut(AppKit.NSWorkspace.getInfoForFile_application_type_, 1)
        self.assertArgIsOut(AppKit.NSWorkspace.getInfoForFile_application_type_, 2)
        self.assertResultIsBOOL(AppKit.NSWorkspace.isFilePackageAtPath_)
        self.assertResultIsBOOL(AppKit.NSWorkspace.setIcon_forFile_options_)
        self.assertResultIsBOOL(
            AppKit.NSWorkspace.getFileSystemInfoForPath_isRemovable_isWritable_isUnmountable_description_type_  # noqa: B950
        )
        self.assertArgHasType(
            AppKit.NSWorkspace.getFileSystemInfoForPath_isRemovable_isWritable_isUnmountable_description_type_,  # noqa: B950
            1,
            b"o^" + objc._C_NSBOOL,
        )
        self.assertArgHasType(
            AppKit.NSWorkspace.getFileSystemInfoForPath_isRemovable_isWritable_isUnmountable_description_type_,  # noqa: B950
            2,
            b"o^" + objc._C_NSBOOL,
        )
        self.assertArgHasType(
            AppKit.NSWorkspace.getFileSystemInfoForPath_isRemovable_isWritable_isUnmountable_description_type_,  # noqa: B950
            3,
            b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsOut(
            AppKit.NSWorkspace.getFileSystemInfoForPath_isRemovable_isWritable_isUnmountable_description_type_,  # noqa: B950
            4,
        )
        self.assertArgIsOut(
            AppKit.NSWorkspace.getFileSystemInfoForPath_isRemovable_isWritable_isUnmountable_description_type_,  # noqa: B950
            5,
        )
        self.assertResultIsBOOL(
            AppKit.NSWorkspace.performFileOperation_source_destination_files_tag_
        )
        self.assertArgIsOut(
            AppKit.NSWorkspace.performFileOperation_source_destination_files_tag_, 4
        )
        self.assertResultIsBOOL(AppKit.NSWorkspace.unmountAndEjectDeviceAtPath_)
        self.assertResultIsBOOL(
            AppKit.NSWorkspace.launchAppWithBundleIdentifier_options_additionalEventParamDescriptor_launchIdentifier_  # noqa: B950
        )
        self.assertArgIsOut(
            AppKit.NSWorkspace.launchAppWithBundleIdentifier_options_additionalEventParamDescriptor_launchIdentifier_,  # noqa: B950
            3,
        )
        self.assertResultIsBOOL(
            AppKit.NSWorkspace.openURLs_withAppBundleIdentifier_options_additionalEventParamDescriptor_launchIdentifiers_  # noqa: B950
        )
        self.assertArgIsOut(
            AppKit.NSWorkspace.openURLs_withAppBundleIdentifier_options_additionalEventParamDescriptor_launchIdentifiers_,  # noqa: B950
            4,
        )
        self.assertArgIsOut(AppKit.NSWorkspace.typeOfFile_error_, 1)
        self.assertResultIsBOOL(AppKit.NSWorkspace.filenameExtension_isValidForType_)
        self.assertResultIsBOOL(AppKit.NSWorkspace.type_conformsToType_)

        self.assertResultIsBOOL(AppKit.NSWorkspace.selectFile_inFileViewerRootedAtPath_)
        self.assertArgHasType(
            AppKit.NSWorkspace.slideImage_from_to_, 1, AppKit.NSPoint.__typestr__
        )
        self.assertArgHasType(
            AppKit.NSWorkspace.slideImage_from_to_, 2, AppKit.NSPoint.__typestr__
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsOut(
            AppKit.NSWorkspace.launchApplicationAtURL_options_configuration_error_, 3
        )
        self.assertResultIsBOOL(AppKit.NSWorkspace.showSearchResultsForQueryString_)

        self.assertArgIsBlock(
            AppKit.NSWorkspace.recycleURLs_completionHandler_, 1, b"v@@"
        )
        self.assertArgIsBlock(
            AppKit.NSWorkspace.duplicateURLs_completionHandler_, 1, b"v@@"
        )

        self.assertResultIsBOOL(AppKit.NSWorkspace.unmountAndEjectDeviceAtURL_error_)
        self.assertArgIsOut(AppKit.NSWorkspace.unmountAndEjectDeviceAtURL_error_, 1)

        self.assertResultIsBOOL(
            AppKit.NSWorkspace.setDesktopImageURL_forScreen_options_error_
        )
        self.assertArgIsOut(
            AppKit.NSWorkspace.setDesktopImageURL_forScreen_options_error_, 3
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsOut(AppKit.NSWorkspace.openURL_options_configuration_error_, 3)
        self.assertArgIsOut(
            AppKit.NSWorkspace.openURLs_withApplicationAtURL_options_configuration_error_,
            4,
        )

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(AppKit.NSWorkspace.isVoiceOverEnabled)
        self.assertResultIsBOOL(AppKit.NSWorkspace.isSwitchControlEnabled)

    @min_os_level("10.14")
    def testMethods10_14(self):
        self.assertArgIsBlock(
            AppKit.NSWorkspace.requestAuthorizationOfType_completionHandler_, 1, b"v@@"
        )

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertArgIsBlock(
            AppKit.NSWorkspace.openURL_configuration_completionHandler_, 2, b"v@@"
        )
        self.assertArgIsBlock(
            AppKit.NSWorkspace.openURLs_withApplicationAtURL_configuration_completionHandler_,
            3,
            b"v@@",
        )
        self.assertArgIsBlock(
            AppKit.NSWorkspace.openApplicationAtURL_configuration_completionHandler_,
            2,
            b"v@@",
        )

        self.assertResultIsBOOL(AppKit.NSWorkspaceOpenConfiguration.promptsUserIfNeeded)
        self.assertResultIsBOOL(AppKit.NSWorkspaceOpenConfiguration.addsToRecentItems)
        self.assertResultIsBOOL(AppKit.NSWorkspaceOpenConfiguration.activates)
        self.assertResultIsBOOL(AppKit.NSWorkspaceOpenConfiguration.hides)
        self.assertResultIsBOOL(AppKit.NSWorkspaceOpenConfiguration.hidesOthers)
        self.assertResultIsBOOL(AppKit.NSWorkspaceOpenConfiguration.isForPrinting)
        self.assertResultIsBOOL(
            AppKit.NSWorkspaceOpenConfiguration.createsNewApplicationInstance
        )
        self.assertResultIsBOOL(
            AppKit.NSWorkspaceOpenConfiguration.requiresUniversalLinks
        )

        self.assertArgIsBOOL(
            AppKit.NSWorkspaceOpenConfiguration.setPromptsUserIfNeeded_, 0
        )
        self.assertArgIsBOOL(
            AppKit.NSWorkspaceOpenConfiguration.setAddsToRecentItems_, 0
        )
        self.assertArgIsBOOL(AppKit.NSWorkspaceOpenConfiguration.setActivates_, 0)
        self.assertArgIsBOOL(AppKit.NSWorkspaceOpenConfiguration.setHides_, 0)
        self.assertArgIsBOOL(AppKit.NSWorkspaceOpenConfiguration.setHidesOthers_, 0)
        self.assertArgIsBOOL(AppKit.NSWorkspaceOpenConfiguration.setForPrinting_, 0)
        self.assertArgIsBOOL(
            AppKit.NSWorkspaceOpenConfiguration.setCreatesNewApplicationInstance_, 0
        )
        self.assertArgIsBOOL(
            AppKit.NSWorkspaceOpenConfiguration.setRequiresUniversalLinks_, 0
        )

    @min_os_level("12.0")
    def testMethods12_0(self):
        self.assertArgIsBlock(
            AppKit.NSWorkspace.setDefaultApplicationAtURL_toOpenContentTypeOfFileAtURL_completionHandler_,
            2,
            b"v@",
        )
        self.assertArgIsBlock(
            AppKit.NSWorkspace.setDefaultApplicationAtURL_toOpenURLsWithScheme_completionHandler_,
            2,
            b"v@",
        )
        self.assertArgIsBlock(
            AppKit.NSWorkspace.setDefaultApplicationAtURL_toOpenFileAtURL_completionHandler_,
            2,
            b"v@",
        )
        self.assertArgIsBlock(
            AppKit.NSWorkspace.setDefaultApplicationAtURL_toOpenContentType_completionHandler_,
            2,
            b"v@",
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSColorPanel(TestCase):
    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertEqual(AppKit.NSNoModeColorPanel, -1)

        self.assertEqual(AppKit.NSColorPanelModeNone, -1)

    def testConstants(self):
        self.assertEqual(AppKit.NSGrayModeColorPanel, 0)
        self.assertEqual(AppKit.NSRGBModeColorPanel, 1)
        self.assertEqual(AppKit.NSCMYKModeColorPanel, 2)
        self.assertEqual(AppKit.NSHSBModeColorPanel, 3)
        self.assertEqual(AppKit.NSCustomPaletteModeColorPanel, 4)
        self.assertEqual(AppKit.NSColorListModeColorPanel, 5)
        self.assertEqual(AppKit.NSWheelModeColorPanel, 6)
        self.assertEqual(AppKit.NSCrayonModeColorPanel, 7)

        self.assertEqual(AppKit.NSColorPanelModeGray, 0)
        self.assertEqual(AppKit.NSColorPanelModeRGB, 1)
        self.assertEqual(AppKit.NSColorPanelModeCMYK, 2)
        self.assertEqual(AppKit.NSColorPanelModeHSB, 3)
        self.assertEqual(AppKit.NSColorPanelModeCustomPalette, 4)
        self.assertEqual(AppKit.NSColorPanelModeColorList, 5)
        self.assertEqual(AppKit.NSColorPanelModeWheel, 6)
        self.assertEqual(AppKit.NSColorPanelModeCrayon, 7)

        self.assertEqual(AppKit.NSColorPanelGrayModeMask, 0x00000001)
        self.assertEqual(AppKit.NSColorPanelRGBModeMask, 0x00000002)
        self.assertEqual(AppKit.NSColorPanelCMYKModeMask, 0x00000004)
        self.assertEqual(AppKit.NSColorPanelHSBModeMask, 0x00000008)
        self.assertEqual(AppKit.NSColorPanelCustomPaletteModeMask, 0x00000010)
        self.assertEqual(AppKit.NSColorPanelColorListModeMask, 0x00000020)
        self.assertEqual(AppKit.NSColorPanelWheelModeMask, 0x00000040)
        self.assertEqual(AppKit.NSColorPanelCrayonModeMask, 0x00000080)
        self.assertEqual(AppKit.NSColorPanelAllModesMask, 0x0000FFFF)

        self.assertIsInstance(AppKit.NSColorPanelColorDidChangeNotification, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSColorPanel.sharedColorPanelExists)
        self.assertResultIsBOOL(AppKit.NSColorPanel.dragColor_withEvent_fromView_)

        self.assertResultIsBOOL(AppKit.NSColorPanel.isContinuous)
        self.assertArgIsBOOL(AppKit.NSColorPanel.setContinuous_, 0)

        self.assertResultIsBOOL(AppKit.NSColorPanel.showsAlpha)
        self.assertArgIsBOOL(AppKit.NSColorPanel.setShowsAlpha_, 0)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSStreamHelper(Foundation.NSObject):
    def stream_handleEvent_(self, a, b):
        pass


class TestNSStreamUsage(TestCase):
    def testUsage(self):
        # Test the usage of methods that require extra work

        # Try to create a connection to the IPP port on the local host
        (
            inputStream,
            outputStream,
        ) = Foundation.NSStream.getStreamsToHost_port_inputStream_outputStream_(
            Foundation.NSHost.hostWithAddress_("127.0.0.1"), 631, None, None  # IPP port
        )

        self.assertIsInstance(inputStream, Foundation.NSInputStream)
        self.assertIsInstance(outputStream, Foundation.NSOutputStream)

        inputStream.close()
        outputStream.close()

    def testConstants(self):
        self.assertEqual(Foundation.NSStreamStatusNotOpen, 0)
        self.assertEqual(Foundation.NSStreamStatusOpening, 1)
        self.assertEqual(Foundation.NSStreamStatusOpen, 2)
        self.assertEqual(Foundation.NSStreamStatusReading, 3)
        self.assertEqual(Foundation.NSStreamStatusWriting, 4)
        self.assertEqual(Foundation.NSStreamStatusAtEnd, 5)
        self.assertEqual(Foundation.NSStreamStatusClosed, 6)
        self.assertEqual(Foundation.NSStreamStatusError, 7)

        self.assertEqual(Foundation.NSStreamEventNone, 0)
        self.assertEqual(Foundation.NSStreamEventOpenCompleted, 1 << 0)
        self.assertEqual(Foundation.NSStreamEventHasBytesAvailable, 1 << 1)
        self.assertEqual(Foundation.NSStreamEventHasSpaceAvailable, 1 << 2)
        self.assertEqual(Foundation.NSStreamEventErrorOccurred, 1 << 3)
        self.assertEqual(Foundation.NSStreamEventEndEncountered, 1 << 4)

        self.assertIsInstance(Foundation.NSStreamSocketSecurityLevelKey, str)
        self.assertIsInstance(Foundation.NSStreamSocketSecurityLevelNone, str)
        self.assertIsInstance(Foundation.NSStreamSocketSecurityLevelSSLv2, str)
        self.assertIsInstance(Foundation.NSStreamSocketSecurityLevelSSLv3, str)
        self.assertIsInstance(Foundation.NSStreamSocketSecurityLevelTLSv1, str)
        self.assertIsInstance(Foundation.NSStreamSocketSecurityLevelNegotiatedSSL, str)
        self.assertIsInstance(Foundation.NSStreamSOCKSProxyConfigurationKey, str)
        self.assertIsInstance(Foundation.NSStreamSOCKSProxyHostKey, str)
        self.assertIsInstance(Foundation.NSStreamSOCKSProxyPortKey, str)
        self.assertIsInstance(Foundation.NSStreamSOCKSProxyVersionKey, str)
        self.assertIsInstance(Foundation.NSStreamSOCKSProxyUserKey, str)
        self.assertIsInstance(Foundation.NSStreamSOCKSProxyPasswordKey, str)
        self.assertIsInstance(Foundation.NSStreamSOCKSProxyVersion4, str)
        self.assertIsInstance(Foundation.NSStreamSOCKSProxyVersion5, str)
        self.assertIsInstance(Foundation.NSStreamDataWrittenToMemoryStreamKey, str)
        self.assertIsInstance(Foundation.NSStreamFileCurrentOffsetKey, str)
        self.assertIsInstance(Foundation.NSStreamSocketSSLErrorDomain, str)
        self.assertIsInstance(Foundation.NSStreamSOCKSErrorDomain, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(Foundation.NSStreamNetworkServiceType, str)
        self.assertIsInstance(Foundation.NSStreamNetworkServiceTypeVoIP, str)
        self.assertIsInstance(Foundation.NSStreamNetworkServiceTypeVideo, str)
        self.assertIsInstance(Foundation.NSStreamNetworkServiceTypeBackground, str)
        self.assertIsInstance(Foundation.NSStreamNetworkServiceTypeVoice, str)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(Foundation.NSStreamNetworkServiceTypeCallSignaling, str)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSStream.setProperty_forKey_)

        self.assertArgHasType(Foundation.NSInputStream.read_maxLength_, 0, b"o^v")
        self.assertArgSizeInArg(Foundation.NSInputStream.read_maxLength_, 0, 1)
        self.assertArgSizeInResult(Foundation.NSInputStream.read_maxLength_, 0)

        self.assertResultIsBOOL(Foundation.NSInputStream.hasBytesAvailable)

        self.assertArgHasType(Foundation.NSOutputStream.write_maxLength_, 0, b"n^v")
        self.assertArgSizeInArg(Foundation.NSOutputStream.write_maxLength_, 0, 1)
        self.assertResultIsBOOL(Foundation.NSOutputStream.hasSpaceAvailable)

        b = Foundation.NSOutputStream.alloc().initToMemory()
        self.assertArgHasType(b.initToFileAtPath_append_, 1, objc._C_NSBOOL)
        self.assertArgHasType(
            Foundation.NSOutputStream.outputStreamToFileAtPath_append_,
            1,
            objc._C_NSBOOL,
        )

        self.assertArgHasType(b.initToBuffer_capacity_, 0, b"o^v")
        self.assertArgSizeInArg(b.initToBuffer_capacity_, 0, 1)
        self.assertArgHasType(
            Foundation.NSOutputStream.outputStreamToBuffer_capacity_, 0, b"o^v"
        )
        self.assertArgSizeInArg(
            Foundation.NSOutputStream.outputStreamToBuffer_capacity_, 0, 1
        )

        self.assertArgIsOut(Foundation.NSInputStream.getBuffer_length_, 0)
        self.assertArgIsOut(Foundation.NSInputStream.getBuffer_length_, 1)
        self.assertArgSizeInArg(Foundation.NSInputStream.getBuffer_length_, 0, 1)

    def testDelegate(self):
        self.assertArgHasType(
            TestNSStreamHelper.stream_handleEvent_, 1, objc._C_NSUInteger
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        b = Foundation.NSOutputStream.alloc()
        try:
            self.assertArgIsBOOL(b.initWithURL_append_, 1)
        finally:
            b = b.initToMemory()
        self.assertArgIsBOOL(Foundation.NSOutputStream.outputStreamWithURL_append_, 1)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsOut(
            Foundation.NSStream.getStreamsToHostWithName_port_inputStream_outputStream_,
            2,
        )
        self.assertArgIsOut(
            Foundation.NSStream.getStreamsToHostWithName_port_inputStream_outputStream_,
            3,
        )

        self.assertArgIsOut(
            Foundation.NSStream.getBoundStreamsWithBufferSize_inputStream_outputStream_,
            1,
        )
        self.assertArgIsOut(
            Foundation.NSStream.getBoundStreamsWithBufferSize_inputStream_outputStream_,
            2,
        )

    @min_sdk_level("10.7")
    def testProtocols(self):
        objc.protocolNamed("NSStreamDelegate")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSDatePicker(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSDatePicker.isBezeled)
        self.assertArgIsBOOL(AppKit.NSDatePicker.setBezeled_, 0)

        self.assertResultIsBOOL(AppKit.NSDatePicker.isBordered)
        self.assertArgIsBOOL(AppKit.NSDatePicker.setBordered_, 0)

        self.assertResultIsBOOL(AppKit.NSDatePicker.drawsBackground)
        self.assertArgIsBOOL(AppKit.NSDatePicker.setDrawsBackground_, 0)

    @min_os_level("10.15.4")
    def testMethods10_15_4(self):
        self.assertResultIsBOOL(AppKit.NSDatePicker.presentsCalendarOverlay)
        self.assertArgIsBOOL(AppKit.NSDatePicker.setPresentsCalendarOverlay_, 0)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSDateInterval(TestCase):
    @min_os_level("10.12")
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSDateInterval.isEqualToDateInterval_)
        self.assertResultIsBOOL(Foundation.NSDateInterval.intersectsDateInterval_)
        self.assertResultIsBOOL(Foundation.NSDateInterval.containsDate_)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSPasteboardItem(TestCase):
    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(AppKit.NSPasteboardItem.setDataProvider_forTypes_)
        self.assertResultIsBOOL(AppKit.NSPasteboardItem.setData_forType_)
        self.assertResultIsBOOL(AppKit.NSPasteboardItem.setString_forType_)
        self.assertResultIsBOOL(AppKit.NSPasteboardItem.setPropertyList_forType_)

    @min_sdk_level("10.6")
    def testProtocols(self):
        objc.protocolNamed("NSPasteboardItemDataProvider")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSDocumentController(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(
            AppKit.NSDocumentController.openUntitledDocumentAndDisplay_error_, 0
        )
        self.assertArgIsOut(
            AppKit.NSDocumentController.openUntitledDocumentAndDisplay_error_, 1
        )
        self.assertArgIsOut(
            AppKit.NSDocumentController.makeUntitledDocumentOfType_error_, 1
        )
        self.assertArgIsBOOL(
            AppKit.NSDocumentController.openDocumentWithContentsOfURL_display_error_, 1
        )
        self.assertArgIsOut(
            AppKit.NSDocumentController.openDocumentWithContentsOfURL_display_error_, 2
        )
        self.assertArgIsOut(
            AppKit.NSDocumentController.makeDocumentWithContentsOfURL_ofType_error_, 2
        )
        self.assertResultIsBOOL(
            AppKit.NSDocumentController.reopenDocumentForURL_withContentsOfURL_error_
        )
        self.assertArgIsOut(
            AppKit.NSDocumentController.reopenDocumentForURL_withContentsOfURL_error_, 2
        )
        self.assertArgIsOut(
            AppKit.NSDocumentController.makeDocumentForURL_withContentsOfURL_ofType_error_,
            3,
        )
        self.assertResultIsBOOL(AppKit.NSDocumentController.hasEditedDocuments)
        self.assertArgIsBOOL(
            AppKit.NSDocumentController.reviewUnsavedDocumentsWithAlertTitle_cancellable_delegate_didReviewAllSelector_contextInfo_,  # noqa: B950
            1,
        )
        self.assertArgIsSEL(
            AppKit.NSDocumentController.reviewUnsavedDocumentsWithAlertTitle_cancellable_delegate_didReviewAllSelector_contextInfo_,  # noqa: B950
            3,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocumentController.reviewUnsavedDocumentsWithAlertTitle_cancellable_delegate_didReviewAllSelector_contextInfo_,  # noqa: B950
            4,
            b"^v",
        )
        self.assertArgIsSEL(
            AppKit.NSDocumentController.closeAllDocumentsWithDelegate_didCloseAllSelector_contextInfo_,  # noqa: B950
            1,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocumentController.closeAllDocumentsWithDelegate_didCloseAllSelector_contextInfo_,  # noqa: B950
            2,
            b"^v",
        )
        self.assertArgIsSEL(
            AppKit.NSDocumentController.presentError_modalForWindow_delegate_didPresentSelector_contextInfo_,  # noqa: B950
            3,
            b"v@:" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocumentController.presentError_modalForWindow_delegate_didPresentSelector_contextInfo_,  # noqa: B950
            4,
            b"^v",
        )
        self.assertResultIsBOOL(AppKit.NSDocumentController.presentError_)
        self.assertArgIsOut(AppKit.NSDocumentController.typeForContentsOfURL_error_, 1)
        self.assertResultIsBOOL(AppKit.NSDocumentController.validateUserInterfaceItem_)
        self.assertArgIsBOOL(
            AppKit.NSDocumentController.openDocumentWithContentsOfFile_display_, 1
        )
        self.assertArgIsBOOL(
            AppKit.NSDocumentController.openDocumentWithContentsOfURL_display_, 1
        )
        self.assertArgIsBOOL(
            AppKit.NSDocumentController.openUntitledDocumentOfType_display_, 1
        )
        self.assertArgIsBOOL(AppKit.NSDocumentController.setShouldCreateUI_, 0)
        self.assertResultIsBOOL(AppKit.NSDocumentController.shouldCreateUI)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsBOOL(
            AppKit.NSDocumentController.openDocumentWithContentsOfURL_display_completionHandler_,  # noqa: B950
            1,
        )
        self.assertArgIsBlock(
            AppKit.NSDocumentController.openDocumentWithContentsOfURL_display_completionHandler_,  # noqa: B950
            2,
            b"v@",
        )
        self.assertArgIsBOOL(
            AppKit.NSDocumentController.reopenDocumentForURL_withContentsOfURL_display_completionHandler_,  # noqa: B950
            2,
        )
        self.assertArgIsBlock(
            AppKit.NSDocumentController.reopenDocumentForURL_withContentsOfURL_display_completionHandler_,  # noqa: B950
            3,
            b"v@" + objc._C_NSBOOL + b"@",
        )
        self.assertArgIsBOOL(
            AppKit.NSDocumentController.duplicateDocumentWithContentsOfURL_copying_displayName_error_,  # noqa: B950
            1,
        )
        self.assertArgIsOut(
            AppKit.NSDocumentController.duplicateDocumentWithContentsOfURL_copying_displayName_error_,  # noqa: B950
            3,
        )

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertArgIsBlock(
            AppKit.NSDocumentController.beginOpenPanelWithCompletionHandler_, 0, b"v@"
        )
        self.assertArgIsBlock(
            AppKit.NSDocumentController.beginOpenPanel_forTypes_completionHandler_,
            2,
            b"v" + objc._C_NSInteger,
        )

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(AppKit.NSDocumentController.allowsAutomaticShareMenu)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSSearchField(TestCase):
    @min_os_level("10.10")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSearchField.sendsWholeSearchString)
        self.assertArgIsBOOL(AppKit.NSSearchField.setSendsWholeSearchString_, 0)
        self.assertResultIsBOOL(AppKit.NSSearchField.sendsSearchStringImmediately)
        self.assertArgIsBOOL(AppKit.NSSearchField.setSendsSearchStringImmediately_, 0)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertArgIsBOOL(AppKit.NSSearchField.rectForSearchTextWhenCentered_, 0)
        self.assertArgIsBOOL(AppKit.NSSearchField.rectForSearchButtonWhenCentered_, 0)
        self.assertArgIsBOOL(AppKit.NSSearchField.rectForCancelButtonWhenCentered_, 0)
        self.assertResultIsBOOL(AppKit.NSSearchField.centersPlaceholder)
        self.assertArgIsBOOL(AppKit.NSSearchField.setCentersPlaceholder_, 0)

    @min_sdk_level("10.11")
    def testProtocols(self):
        objc.protocolNamed("NSSearchFieldDelegate")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSComboBoxHelper(AppKit.NSObject):
    def numberOfItemsInComboBox_(self, b):
        return 1

    def comboBox_objectValueForItemAtIndex_(self, b, i):
        return 1

    def comboBox_indexOfItemWithStringValue_(self, b, s):
        return 1


class TestNSComboBox(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSComboBoxWillPopUpNotification, str)
        self.assertIsInstance(AppKit.NSComboBoxWillDismissNotification, str)
        self.assertIsInstance(AppKit.NSComboBoxSelectionDidChangeNotification, str)
        self.assertIsInstance(AppKit.NSComboBoxSelectionIsChangingNotification, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSComboBox.hasVerticalScroller)
        self.assertArgIsBOOL(AppKit.NSComboBox.setHasVerticalScroller_, 0)
        self.assertResultIsBOOL(AppKit.NSComboBox.isButtonBordered)
        self.assertArgIsBOOL(AppKit.NSComboBox.setButtonBordered_, 0)
        self.assertResultIsBOOL(AppKit.NSComboBox.usesDataSource)
        self.assertArgIsBOOL(AppKit.NSComboBox.setUsesDataSource_, 0)
        self.assertResultIsBOOL(AppKit.NSComboBox.completes)
        self.assertArgIsBOOL(AppKit.NSComboBox.setCompletes_, 0)

    @min_os_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSComboBoxDataSource")
        objc.protocolNamed("NSComboBoxDelegate")

    def testProtocols(self):
        self.assertResultHasType(
            TestNSComboBoxHelper.numberOfItemsInComboBox_, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSComboBoxHelper.comboBox_objectValueForItemAtIndex_,
            1,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSComboBoxHelper.comboBox_indexOfItemWithStringValue_,
            objc._C_NSUInteger,
        )



import AppKit
from PyObjCTools.TestSupport import (
    TestCase,
    min_os_level,
    min_sdk_level,
    expectedFailure,
    os_release,
    expectedFailureIf,
)
import objc


class TestNSLayoutManagerHelper(AppKit.NSObject):
    def layoutManager_shouldUseTemporaryAttributes_forDrawingToScreen_atCharacterIndex_effectiveRange_(  # noqa: B950
        self, a, b, c, d, e
    ):
        return 1

    def layoutManager_didCompleteLayoutForTextContainer_atEnd_(self, a, b, c):
        return 1

    def layoutOrientation(self):
        return 1

    def layoutManager_shouldGenerateGlyphs_properties_characterIndexes_forGlyphRange_(
        self, x, g, p, c, r
    ):
        pass

    def layoutManager_lineSpacingAfterGlyphAtIndex_withProposedLineFragmentRect_(
        self, x, i, r
    ):
        return 1

    def layoutManager_paragraphSpacingBeforeGlyphAtIndex_withProposedLineFragmentRect_(
        self, x, i, r
    ):
        return 1

    def layoutManager_paragraphSpacingAfterGlyphAtIndex_withProposedLineFragmentRect_(
        self, x, i, r
    ):
        return 1

    def layoutManager_shouldUseAction_forControlCharacterAtIndex_(self, x, a, i):
        return 1

    def layoutManager_shouldBreakLineByWordBeforeCharacterAtIndex_(self, x, i):
        return 1

    def layoutManager_shouldBreakLineByHyphenatingBeforeCharacterAtIndex_(self, x, i):
        return 1

    def layoutManager_boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_(  # noqa: B950
        self, x, i, c, f, p, i2
    ):
        return 1

    def layoutManager_textContainer_didChangeGeometryFromSize_(self, x, c, s):
        pass

    def layoutManager_shouldSetLineFragmentRect_lineFragmentUsedRect_baselineOffset_inTextContainer_forGlyphRange_(  # noqa: B950
        self, x, fr, ur, o, c, r
    ):
        return 1


class TestNSLayoutManager(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSGlyphAttributeSoft, 0)
        self.assertEqual(AppKit.NSGlyphAttributeElastic, 1)
        self.assertEqual(AppKit.NSGlyphAttributeBidiLevel, 2)
        self.assertEqual(AppKit.NSGlyphAttributeInscribe, 5)

        self.assertEqual(AppKit.NSGlyphInscribeBase, 0)
        self.assertEqual(AppKit.NSGlyphInscribeBelow, 1)
        self.assertEqual(AppKit.NSGlyphInscribeAbove, 2)
        self.assertEqual(AppKit.NSGlyphInscribeOverstrike, 3)
        self.assertEqual(AppKit.NSGlyphInscribeOverBelow, 4)

        self.assertEqual(AppKit.NSTypesetterLatestBehavior, -1)
        self.assertEqual(AppKit.NSTypesetterOriginalBehavior, 0)
        self.assertEqual(AppKit.NSTypesetterBehavior_10_2_WithCompatibility, 1)
        self.assertEqual(AppKit.NSTypesetterBehavior_10_2, 2)
        self.assertEqual(AppKit.NSTypesetterBehavior_10_3, 3)
        self.assertEqual(AppKit.NSTypesetterBehavior_10_4, 4)

        self.assertEqual(AppKit.NSTextLayoutOrientationHorizontal, 0)
        self.assertEqual(AppKit.NSTextLayoutOrientationVertical, 1)

        # OSX 10.11:
        self.assertEqual(AppKit.NSGlyphPropertyNull, 1 << 0)
        self.assertEqual(AppKit.NSGlyphPropertyControlCharacter, 1 << 1)
        self.assertEqual(AppKit.NSGlyphPropertyElastic, 1 << 2)
        self.assertEqual(AppKit.NSGlyphPropertyNonBaseCharacter, 1 << 3)

        self.assertEqual(AppKit.NSControlCharacterActionZeroAdvancement, 1 << 0)
        self.assertEqual(AppKit.NSControlCharacterActionWhitespace, 1 << 1)
        self.assertEqual(AppKit.NSControlCharacterActionHorizontalTab, 1 << 2)
        self.assertEqual(AppKit.NSControlCharacterActionLineBreak, 1 << 3)
        self.assertEqual(AppKit.NSControlCharacterActionParagraphBreak, 1 << 4)
        self.assertEqual(AppKit.NSControlCharacterActionContainerBreak, 1 << 5)

    @min_sdk_level("10.7")
    def testProtocolObjects(self):
        objc.protocolNamed("NSTextLayoutOrientationProvider")
        objc.protocolNamed("NSLayoutManagerDelegate")

    def testProtocols(self):
        self.assertResultHasType(
            TestNSLayoutManagerHelper.layoutOrientation, objc._C_NSInteger
        )

        # XXX: check interface!
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldGenerateGlyphs_properties_characterIndexes_forGlyphRange_,  # noqa: B950
            1,
            b"n^" + objc._C_USHT,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldGenerateGlyphs_properties_characterIndexes_forGlyphRange_,  # noqa: B950
            2,
            b"n^" + objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldGenerateGlyphs_properties_characterIndexes_forGlyphRange_,  # noqa: B950
            3,
            b"n^" + objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldGenerateGlyphs_properties_characterIndexes_forGlyphRange_,  # noqa: B950
            4,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgSizeInArg(
            TestNSLayoutManagerHelper.layoutManager_shouldGenerateGlyphs_properties_characterIndexes_forGlyphRange_,  # noqa: B950
            1,
            4,
        )
        self.assertArgSizeInArg(
            TestNSLayoutManagerHelper.layoutManager_shouldGenerateGlyphs_properties_characterIndexes_forGlyphRange_,  # noqa: B950
            2,
            4,
        )
        self.assertArgSizeInArg(
            TestNSLayoutManagerHelper.layoutManager_shouldGenerateGlyphs_properties_characterIndexes_forGlyphRange_,  # noqa: B950
            3,
            4,
        )

        self.assertResultHasType(
            TestNSLayoutManagerHelper.layoutManager_lineSpacingAfterGlyphAtIndex_withProposedLineFragmentRect_,  # noqa: B950
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_lineSpacingAfterGlyphAtIndex_withProposedLineFragmentRect_,  # noqa: B950
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_lineSpacingAfterGlyphAtIndex_withProposedLineFragmentRect_,  # noqa: B950
            2,
            AppKit.NSRect.__typestr__,
        )

        self.assertResultHasType(
            TestNSLayoutManagerHelper.layoutManager_paragraphSpacingBeforeGlyphAtIndex_withProposedLineFragmentRect_,  # noqa: B950
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_paragraphSpacingBeforeGlyphAtIndex_withProposedLineFragmentRect_,  # noqa: B950
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_paragraphSpacingBeforeGlyphAtIndex_withProposedLineFragmentRect_,  # noqa: B950
            2,
            AppKit.NSRect.__typestr__,
        )

        self.assertResultHasType(
            TestNSLayoutManagerHelper.layoutManager_paragraphSpacingAfterGlyphAtIndex_withProposedLineFragmentRect_,  # noqa: B950
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_paragraphSpacingAfterGlyphAtIndex_withProposedLineFragmentRect_,  # noqa: B950
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_paragraphSpacingAfterGlyphAtIndex_withProposedLineFragmentRect_,  # noqa: B950
            2,
            AppKit.NSRect.__typestr__,
        )

        self.assertResultHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldUseAction_forControlCharacterAtIndex_,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldUseAction_forControlCharacterAtIndex_,
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldUseAction_forControlCharacterAtIndex_,
            2,
            objc._C_NSInteger,
        )

        self.assertResultHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldBreakLineByWordBeforeCharacterAtIndex_,  # noqa: B950
            objc._C_NSBOOL,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldBreakLineByWordBeforeCharacterAtIndex_,  # noqa: B950
            1,
            objc._C_NSUInteger,
        )

        self.assertResultHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldBreakLineByHyphenatingBeforeCharacterAtIndex_,  # noqa: B950
            objc._C_NSBOOL,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldBreakLineByWordBeforeCharacterAtIndex_,  # noqa: B950
            1,
            objc._C_NSUInteger,
        )

        self.assertResultHasType(
            TestNSLayoutManagerHelper.layoutManager_boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            3,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            4,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            5,
            objc._C_NSUInteger,
        )

        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_textContainer_didChangeGeometryFromSize_,
            2,
            AppKit.NSSize.__typestr__,
        )

        self.assertResultIsBOOL(
            TestNSLayoutManagerHelper.layoutManager_shouldSetLineFragmentRect_lineFragmentUsedRect_baselineOffset_inTextContainer_forGlyphRange_  # noqa: B950
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldSetLineFragmentRect_lineFragmentUsedRect_baselineOffset_inTextContainer_forGlyphRange_,  # noqa: B950
            1,
            b"N^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldSetLineFragmentRect_lineFragmentUsedRect_baselineOffset_inTextContainer_forGlyphRange_,  # noqa: B950
            2,
            b"N^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldSetLineFragmentRect_lineFragmentUsedRect_baselineOffset_inTextContainer_forGlyphRange_,  # noqa: B950
            3,
            b"N^" + objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldSetLineFragmentRect_lineFragmentUsedRect_baselineOffset_inTextContainer_forGlyphRange_,  # noqa: B950
            5,
            AppKit.NSRange.__typestr__,
        )

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSLayoutManager.usesScreenFonts)
        self.assertArgIsBOOL(AppKit.NSLayoutManager.setUsesScreenFonts_, 0)

        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.insertGlyphs_length_forStartingGlyphAtIndex_characterIndex_,
            0,
            1,
        )
        self.assertArgIsIn(
            AppKit.NSLayoutManager.insertGlyphs_length_forStartingGlyphAtIndex_characterIndex_,  # noqa: B950
            0,
        )

        self.assertArgIsOut(
            AppKit.NSLayoutManager.invalidateGlyphsForCharacterRange_changeInLength_actualCharacterRange_,  # noqa: B950
            2,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.invalidateLayoutForCharacterRange_isSoft_actualCharacterRange_,  # noqa: B950
            2,
        )
        self.assertArgIsOut(AppKit.NSLayoutManager.getGlyphs_range_, 0)
        self.assertArgSizeInArg(AppKit.NSLayoutManager.getGlyphs_range_, 0, 1)

        self.assertResultIsBOOL(AppKit.NSLayoutManager.showsInvisibleCharacters)
        self.assertArgIsBOOL(AppKit.NSLayoutManager.setShowsInvisibleCharacters_, 0)

        self.assertResultIsBOOL(AppKit.NSLayoutManager.showsControlCharacters)
        self.assertArgIsBOOL(AppKit.NSLayoutManager.setShowsControlCharacters_, 0)

        self.assertResultIsBOOL(AppKit.NSLayoutManager.usesFontLeading)
        self.assertArgIsBOOL(AppKit.NSLayoutManager.setUsesFontLeading_, 0)

        self.assertResultIsBOOL(AppKit.NSLayoutManager.backgroundLayoutEnabled)
        self.assertArgIsBOOL(AppKit.NSLayoutManager.setBackgroundLayoutEnabled_, 0)

        self.assertArgIsOut(
            AppKit.NSLayoutManager.textContainerForGlyphAtIndex_effectiveRange_, 1
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.lineFragmentRectForGlyphAtIndex_effectiveRange_, 1
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.lineFragmentUsedRectForGlyphAtIndex_effectiveRange_,
            1,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.layoutRectForTextBlock_atIndex_effectiveRange_, 2
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.boundsRectForTextBlock_atIndex_effectiveRange_, 2
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.glyphRangeForCharacterRange_actualCharacterRange_, 1
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.characterRangeForGlyphRange_actualGlyphRange_, 1
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.temporaryAttributesAtCharacterIndex_effectiveRange_,
            1,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.lineFragmentRectForGlyphAtIndex_effectiveRange_withoutAdditionalLayout_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            AppKit.NSLayoutManager.lineFragmentRectForGlyphAtIndex_effectiveRange_withoutAdditionalLayout_,  # noqa: B950
            2,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.lineFragmentUsedRectForGlyphAtIndex_effectiveRange_withoutAdditionalLayout_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            AppKit.NSLayoutManager.lineFragmentUsedRectForGlyphAtIndex_effectiveRange_withoutAdditionalLayout_,  # noqa: B950
            2,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.textContainerForGlyphAtIndex_effectiveRange_withoutAdditionalLayout_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            AppKit.NSLayoutManager.textContainerForGlyphAtIndex_effectiveRange_withoutAdditionalLayout_,  # noqa: B950
            2,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getFirstUnlaidCharacterIndex_glyphIndex_, 0
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getFirstUnlaidCharacterIndex_glyphIndex_, 1
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.glyphIndexForPoint_inTextContainer_fractionOfDistanceThroughGlyph_,  # noqa: B950
            2,
        )

        self.assertResultSizeInArg(
            AppKit.NSLayoutManager.rectArrayForCharacterRange_withinSelectedCharacterRange_inTextContainer_rectCount_,  # noqa: B950
            3,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.rectArrayForCharacterRange_withinSelectedCharacterRange_inTextContainer_rectCount_,  # noqa: B950
            3,
        )
        self.assertResultSizeInArg(
            AppKit.NSLayoutManager.rectArrayForGlyphRange_withinSelectedGlyphRange_inTextContainer_rectCount_,  # noqa: B950
            3,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.rectArrayForGlyphRange_withinSelectedGlyphRange_inTextContainer_rectCount_,  # noqa: B950
            3,
        )

        self.assertArgIsOut(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_,  # noqa: B950
            1,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_,  # noqa: B950
            1,
            0,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_,  # noqa: B950
            2,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_,  # noqa: B950
            2,
            0,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_,  # noqa: B950
            3,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_,  # noqa: B950
            3,
            0,
        )
        self.assertArgHasType(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_,  # noqa: B950
            4,
            b"o^" + objc._C_NSBOOL,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_,  # noqa: B950
            4,
            0,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            1,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            1,
            0,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            2,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            2,
            0,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            3,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            3,
            0,
        )
        self.assertArgHasType(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            4,
            b"o^" + objc._C_NSBOOL,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            4,
            0,
        )
        self.assertArgHasType(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            5,
            b"o^" + objc._C_UCHR,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            5,
            0,
        )
        self.assertArgHasType(
            AppKit.NSLayoutManager.showPackedGlyphs_length_glyphRange_atPoint_font_color_printingAdjustment_,  # noqa: B950
            0,
            b"n^v",
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.showPackedGlyphs_length_glyphRange_atPoint_font_color_printingAdjustment_,  # noqa: B950
            0,
            1,
        )

        self.assertResultIsBOOL(AppKit.NSLayoutManager.isValidGlyphIndex_)

        self.assertArgIsBOOL(
            AppKit.NSLayoutManager.setNotShownAttribute_forGlyphAtIndex_, 0
        )
        self.assertArgIsBOOL(
            AppKit.NSLayoutManager.setDrawsOutsideLineFragment_forGlyphAtIndex_, 0
        )

        self.assertResultIsBOOL(
            AppKit.NSLayoutManager.notShownAttributeForGlyphAtIndex_
        )
        self.assertResultIsBOOL(
            AppKit.NSLayoutManager.drawsOutsideLineFragmentForGlyphAtIndex_
        )

    @min_os_level("10.11")
    def testMethods_missing_10_10(self):
        # Document, but not present on 10.10??
        self.assertArgHasType(
            AppKit.NSLayoutManager.CGGlyphAtIndex_isValidIndex_, 1, b"o^Z"
        )

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(AppKit.NSLayoutManager.allowsNonContiguousLayout)
        self.assertArgIsBOOL(AppKit.NSLayoutManager.setAllowsNonContiguousLayout_, 0)

        self.assertResultIsBOOL(AppKit.NSLayoutManager.hasNonContiguousLayout)

        self.assertArgIsOut(
            AppKit.NSLayoutManager.invalidateGlyphsForCharacterRange_changeInLength_actualCharacterRange_,  # noqa: B950
            2,
        )
        self.assertArgHasType(
            AppKit.NSLayoutManager.glyphAtIndex_isValidIndex_, 1, b"o^" + objc._C_NSBOOL
        )

        self.assertArgIsIn(
            AppKit.NSLayoutManager.setLocations_startingGlyphIndexes_count_forGlyphRange_,
            0,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.setLocations_startingGlyphIndexes_count_forGlyphRange_,
            0,
            2,
        )
        self.assertArgIsIn(
            AppKit.NSLayoutManager.setLocations_startingGlyphIndexes_count_forGlyphRange_,
            1,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.setLocations_startingGlyphIndexes_count_forGlyphRange_,
            1,
            2,
        )

        self.assertArgIsOut(
            AppKit.NSLayoutManager.temporaryAttribute_atCharacterIndex_effectiveRange_,
            2,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.temporaryAttribute_atCharacterIndex_longestEffectiveRange_inRange_,  # noqa: B950
            2,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.temporaryAttributesAtCharacterIndex_longestEffectiveRange_inRange_,  # noqa: B950
            1,
        )

        self.assertArgIsBOOL(
            TestNSLayoutManagerHelper.layoutManager_shouldUseTemporaryAttributes_forDrawingToScreen_atCharacterIndex_effectiveRange_,  # noqa: B950
            2,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldUseTemporaryAttributes_forDrawingToScreen_atCharacterIndex_effectiveRange_,  # noqa: B950
            3,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSLayoutManagerHelper.layoutManager_shouldUseTemporaryAttributes_forDrawingToScreen_atCharacterIndex_effectiveRange_,  # noqa: B950
            4,
            b"o^" + AppKit.NSRange.__typestr__,
        )

        self.assertArgIsBOOL(
            TestNSLayoutManagerHelper.layoutManager_didCompleteLayoutForTextContainer_atEnd_,  # noqa: B950
            2,
        )

        self.assertArgIsBOOL(
            AppKit.NSLayoutManager.getLineFragmentInsertionPointsForCharacterAtIndex_alternatePositions_inDisplayOrder_positions_characterIndexes_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            AppKit.NSLayoutManager.getLineFragmentInsertionPointsForCharacterAtIndex_alternatePositions_inDisplayOrder_positions_characterIndexes_,  # noqa: B950
            2,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getLineFragmentInsertionPointsForCharacterAtIndex_alternatePositions_inDisplayOrder_positions_characterIndexes_,  # noqa: B950
            3,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getLineFragmentInsertionPointsForCharacterAtIndex_alternatePositions_inDisplayOrder_positions_characterIndexes_,  # noqa: B950
            4,
        )

        self.assertResultIsBOOL(
            AppKit.NSLayoutManager.layoutManagerOwnsFirstResponderInWindow_
        )

    @expectedFailure
    def testMethods10_5_failon11beta(self):
        self.assertArgIsBOOL(
            AppKit.NSLayoutManager.rulerAccessoryViewForTextView_paragraphStyle_ruler_enabled_,
            3,
        )

    @expectedFailureIf(
        os_release().rsplit(".", 1)[0] in ("10.5", "10.6", "10.7", "10.8", "10.9")
    )
    @min_os_level("10.5")
    def testMethods10_5_not_available(self):
        self.assertHasAttr(
            AppKit.NSLayoutManager,
            "getGlyphsInRange_glyphs_properties_characterIndexes_bidiLevels_",
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_properties_characterIndexes_bidiLevels_,  # noqa: B950
            1,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_properties_characterIndexes_bidiLevels_,  # noqa: B950
            2,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_properties_characterIndexes_bidiLevels_,  # noqa: B950
            3,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_properties_characterIndexes_bidiLevels_,  # noqa: B950
            4,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_properties_characterIndexes_bidiLevels_,  # noqa: B950
            1,
            0,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_properties_characterIndexes_bidiLevels_,  # noqa: B950
            2,
            0,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_properties_characterIndexes_bidiLevels_,  # noqa: B950
            3,
            0,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.getGlyphsInRange_glyphs_properties_characterIndexes_bidiLevels_,  # noqa: B950
            4,
            0,
        )

    @min_os_level("10.5")
    @expectedFailure
    def testMethods10_5_fail(self):
        self.fail(
            "Buffer size is non-trivial: - (AppKit.NSUInteger)getLineFragmentInsertionPointsForCharacterAtIndex:(AppKit.NSUInteger)charIndex alternatePositions:(BOOL)aFlag inDisplayOrder:(BOOL)dFlag positions:(CGFloat *)positions characterIndexes:(AppKit.NSUInteger *)charIndexes;"  # noqa: B950
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgHasType(
            AppKit.NSLayoutManager.characterIndexForPoint_inTextContainer_fractionOfDistanceBetweenInsertionPoints_,  # noqa: B950
            0,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgIsOut(
            AppKit.NSLayoutManager.characterIndexForPoint_inTextContainer_fractionOfDistanceBetweenInsertionPoints_,  # noqa: B950
            2,
        )

        self.assertArgHasType(
            AppKit.NSLayoutManager.fillBackgroundRectArray_count_forCharacterRange_color_,
            0,
            b"N^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.fillBackgroundRectArray_count_forCharacterRange_color_,
            0,
            1,
        )
        self.assertArgHasType(
            AppKit.NSLayoutManager.fillBackgroundRectArray_count_forCharacterRange_color_,
            2,
            AppKit.NSRange.__typestr__,
        )

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsIn(
            AppKit.NSLayoutManager.showCGGlyphs_positions_count_font_matrix_attributes_inContext_,  # noqa: B950
            0,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.showCGGlyphs_positions_count_font_matrix_attributes_inContext_,  # noqa: B950
            0,
            2,
        )
        self.assertArgIsIn(
            AppKit.NSLayoutManager.showCGGlyphs_positions_count_font_matrix_attributes_inContext_,  # noqa: B950
            1,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.showCGGlyphs_positions_count_font_matrix_attributes_inContext_,  # noqa: B950
            1,
            2,
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.setGlyphs_properties_characterIndexes_font_forGlyphRange_,
            0,
            4,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.setGlyphs_properties_characterIndexes_font_forGlyphRange_,
            1,
            4,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.setGlyphs_properties_characterIndexes_font_forGlyphRange_,
            2,
            4,
        )

        self.assertArgIsBlock(
            AppKit.NSLayoutManager.enumerateLineFragmentsForGlyphRange_usingBlock_,
            1,
            b"v"
            + AppKit.NSRect.__typestr__
            + AppKit.NSRect.__typestr__
            + b"@"
            + AppKit.NSRange.__typestr__
            + b"o^Z",
        )
        self.assertArgIsBlock(
            AppKit.NSLayoutManager.enumerateEnclosingRectsForGlyphRange_withinSelectedGlyphRange_inTextContainer_usingBlock_,  # noqa: B950
            3,
            b"v" + AppKit.NSRect.__typestr__ + b"o^Z",
        )

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertResultIsBOOL(AppKit.NSLayoutManager.usesDefaultHyphenation)
        self.assertArgIsBOOL(AppKit.NSLayoutManager.setUsesDefaultHyphenation_, 0)

        self.assertArgIsIn(
            AppKit.NSLayoutManager.showCGGlyphs_positions_count_font_textMatrix_attributes_inContext_,  # noqa: B950
            0,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.showCGGlyphs_positions_count_font_textMatrix_attributes_inContext_,  # noqa: B950
            0,
            2,
        )
        self.assertArgIsIn(
            AppKit.NSLayoutManager.showCGGlyphs_positions_count_font_textMatrix_attributes_inContext_,  # noqa: B950
            1,
        )
        self.assertArgSizeInArg(
            AppKit.NSLayoutManager.showCGGlyphs_positions_count_font_textMatrix_attributes_inContext_,  # noqa: B950
            1,
            2,
        )



#
#  OverdueTransformer.py
#  ToDos
#
#  Converted by u.fiedler on 09.02.05.
#
#  The original version was written in Objective-C by Malcolm Crawford
#  at http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

from Cocoa import NSColor, NSValueTransformer


class OverdueTransformer(NSValueTransformer):
    @classmethod
    def transformedValueClass(cls):
        return NSColor

    @classmethod
    def allowsReverseTransformation(cls):
        return False

    def transformedValue_(self, aDate):
        if aDate is None:
            return None
        if aDate.timeIntervalSinceNow() < 0:
            return NSColor.redColor()
        return NSColor.blackColor()



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSStepperCell(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSStepperCell.valueWraps)
        self.assertArgIsBOOL(AppKit.NSStepperCell.setValueWraps_, 0)
        self.assertResultIsBOOL(AppKit.NSStepperCell.autorepeat)
        self.assertArgIsBOOL(AppKit.NSStepperCell.setAutorepeat_, 0)
